from pathlib import Path
from typing import Dict, List

from pydantic import validator

from .. import Annotation, Request


class TextRequest(Request):
    """
    Request representing a single piece of text, optionally with associated markup

    For example a list of paragraphs or sentences, or a corpus of documents, each divided into sentences.
    While this could be represented as standoff annotations in a plain "text" request, the structured format is more
    suitable for certain types of tools.

    Attributes
    ----------
    type (str, required): the type of request. must be "text"
    params (Dict, optional): vendor specific requirements
    content: (str, optional) text content
    mimeType: (str, optional) mime type of request, default text/plain
    features (Dict, optional): arbitrary json metadata about content
    annotations (Dict[str, List[Annotation]], optional): optional annotations on request
    """

    type: str = "text"
    content: str
    mimeType: str = "text/plain"
    features: Dict = None
    annotations: Dict[str, List[Annotation]] = None

    @classmethod
    def from_file(cls, filename, features: Dict = None, annotations: Dict[str, List[Annotation]] = None):
        filename = Path(filename)
        if not filename.is_file():
            raise ValueError(f"{filename} musts be the path to a file.")
        with open(filename) as f:
            content = f.read()
        return cls(
            content=content,
            features=features,
            annotations=annotations,
        )



from __future__ import annotations

from typing import Dict, List, Text

from pydantic import BaseModel, root_validator

from .. import Annotation, Request


class Text(BaseModel):
    """
    A single node in a structured text request.

    Each text can have an associated score (for example a confidence value for multiple alternative translations or
    transcriptions) and optional annotations, which can be linked to both the result text in this object and to the
    original source material from the corresponding request.

    Attributes
    ----------
    content: (str, optional) text content
    mime_type: (str, optional) mime type of request, default text/plain
    features (dict, optional) arbitrary json metadata about content
    annotations (Dict[str, List[Annotation]], optional): optional annotations on request
    texts: (List[Text], optional): recursive, same structure again
    """

    content: str = None
    mime_type: str = "text/plain"
    features: dict = None
    annotations: Dict[str, List[Annotation]] = None
    texts: List[Text] = None

    @root_validator()
    def either_content_or_text(cls, values):
        """
        validator: ensures only either the "content" or the "text" fields are present
        """
        content, texts = values.get("content"), values.get("texts")
        if content is None  and texts is None:
            raise ValueError('A structured text request must have either "content" or "texts" fields')
        if content:
            values["mimeType"] = "text/plain"
        else:
            values["mimeType"] = None
        return values


class StructuredTextRequest(Request):
    """
    Request representing text with some structure,

    For example a list of paragraphs or sentences, or a corpus of documents, each divided into sentences.
    While this could be represented as standoff annotations in a plain "text" request, the structured format is more
    suitable for certain types of tools.

    Attributes
    ----------
    type (str, required): the type of request. must be "audio"
    params (dict, optional): vendor specific requirements
    texts: (List[Text], required): the actual text object with the text content
    """

    type: str = "structuredText"
    texts: List[Text]



from abc import ABC, abstractmethod
from argparse import ArgumentParser


class BaseELGCommand(ABC):
    @staticmethod
    @abstractmethod
    def register_subcommand(parser: ArgumentParser):
        raise NotImplementedError()

    @abstractmethod
    def run(self):
        raise NotImplementedError()



import argparse
import sys

from loguru import logger
from requests import HTTPError

from .. import __version__
from .docker import DockerCommand
from .download import DownloadCommand
from .info import InfoCommand
from .run import RunCommand
from .search import SearchCommand


def main():
    parser = argparse.ArgumentParser(
        "ELG CLI",
        usage="elg <command> [<args>]",
        epilog="For more information about a command, run: `elg <command> --help`",
    )
    parser.add_argument("--version", "-v", help="Display ELG version", action="store_true")
    commands_parser = parser.add_subparsers(help="commands")

    # Register commands
    RunCommand.register_subcommand(commands_parser)
    DownloadCommand.register_subcommand(commands_parser)
    InfoCommand.register_subcommand(commands_parser)
    SearchCommand.register_subcommand(commands_parser)
    DockerCommand.register_subcommand(commands_parser)

    args = parser.parse_args()

    if args.version:
        print(__version__)
        exit(0)

    if not hasattr(args, "func"):
        parser.print_help()
        exit(1)

    command = args.func(args)

    try:
        command.run()
    except Exception as e:
        logger.error(f"❌ Something failed during the command run - {e}")
        sys.exit(1)


if __name__ == "__main__":
    main()



from pydantic import BaseModel

from .ResponseObject import ResponseObject


class Response(BaseModel):
    """
    Representation of a successful completion response.

    Abstract, subclasses should instantiate this with their own type

    Attributes
    ----------
    response (ResponseObject, required):
    """

    response: ResponseObject



import requests


# main ELG exception
class ElgException(Exception):
    def __init__(self, message):
        super().__init__(message)


"""
Request exceptions
"""

# decorator to catch requests errors and raise Elg errors
def catch_requests_error(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except requests.exceptions.Timeout as e:
            raise TimeoutException(f"{e}")
        except requests.exceptions.RequestException as e:
            raise ConnectException(f"{e}")
        except Exception as e:
            raise e

    return wrapper


def ensure_response_ok(response):
    if response.ok:
        return True
    try:
        r = response.json()
    except:
        try:
            r = response.content
        except:
            r = ""
    if 400 <= response.status_code < 500:
        raise ClientException(r, response.status_code)
    if 500 <= response.status_code:
        raise ServerException(r, response.status_code)
    raise ElgException(r)


# we haven't been able to call the API
class ConnectException(ElgException):
    def __init__(self, error):
        self.message = f"Impossible to connect to the API (Caused by {error})"
        super().__init__(self.message)


# we have been able to call the API, but it responded with an error
class APIException(ElgException):
    def __init__(self, message):
        super().__init__(message)


# 4xx errors with the request the client sent
class ClientException(APIException):
    def __init__(self, response, status_code):
        self.message = f"Error {status_code} with the client request: {response}"
        super().__init__(self.message)


# 5xx errors that occurred while processing the request
class ServerException(APIException):
    def __init__(self, response, status_code):
        self.message = f"Error {status_code} on the server side: {response}"
        super().__init__(self.message)


# Timeout on the request
class TimeoutException(APIException):
    def __init__(self, error):
        self.message = f"Timeout on the API request (Caused by {error})"
        super().__init__(self.message)


"""
User exceptions
"""

# the id used to load a service doesn't correspond to a service
class NotServiceException(ElgException):
    def __init__(self, id):
        self.message = f"The entity with id={id} is not a technology."
        super().__init__(self.message)


# the id used to load a corpus doesn't correspond to a corpus
class NotCorpusException(ElgException):
    def __init__(self, id):
        self.message = f"The entity with id={id} is not a corpus."
        super().__init__(self.message)


# needed information is not in the metadata
class MissingInformationException(ElgException):
    def __init__(self, id, information):
        if id > 0:
            self.message = f"The entity with id={id} does not have the information: {information}."
        else:
            self.message = f"The entity does not have the information: {information}."
        super().__init__(self.message)


# use expired refresh token
class RefreshTokenExpirationException(ElgException):
    def __init__(self, response=None):
        if not response:
            self.message = "The refresh token expired. You need to re-authenticate."
        else:
            self.message = response
        super().__init__(self.message)


class AuthenticationException(ElgException):
    def __init__(self, response=None):
        if not response:
            self.message = (
                "You are not authenticated. Please run `Authentication.init(scope, domain)` or load tokens "
                "using `Authentication.from_json(filename)`."
            )
        else:
            self.message = response
        super().__init__(self.message)


# domains conflict between 2 arguments
class DomainException(ElgException):
    def __init__(self, domain, auth_domain):
        self.message = f"The domain ({domain}) is different from the domain of the auth_file ({auth_domain})."
        super().__init__(self.message)



import hashlib
import io
import json
import pickle
import time
import warnings
from functools import wraps
from os import makedirs
from os.path import expanduser, isfile, splitext
from pathlib import Path
from time import sleep
from typing import Callable, Dict, List, Union
from urllib.parse import urlparse

import requests

from .authentication import (ONE_HUNDRED_YEARS_IN_SECONDS, Authentication,
                             NeedAuthentication, need_authentication)
from .entity import Entity
from .model import (AudioRequest, Request, StructuredTextRequest, TextRequest,
                    get_response)
from .utils import (MIME, get_argument_from_json, get_domain, get_information,
                    get_metadatarecord, map_metadatarecord_to_result)
from .utils.errors import (DomainException, NotServiceException,
                           RefreshTokenExpirationException,
                           catch_requests_error, ensure_response_ok)


class Service(Entity, NeedAuthentication):
    """
    Class to use ELG service. Run an ELG service directly from python.

    Examples::

        from elg import Service

        # You can initialize a service from its id. You will be asked to authenticate on the ELG website.
        service = Service.from_id(474)

        # You can then directly run the service.
        result = service("Nikolas Tesla lives in Berlin.")
        print(f"\\nResult:\\n{result}")

        # You can also create a service from a catalog search result.

        # First you need to search for a service using the catalog. Let's search an English to French Machine Translation service.
        from elg import Catalog

        catalog = Catalog()
        results = catalog.search(
            resource = "Tool/Service",
            function = "Machine Translation",
            languages = ["en", "fr"],
            limit = 1,
        )

        # Now you can initialize the service using the first result. You will not be asked to authenticate because your token has been cached.
        service = Service.from_entity(results[0])

        # And run the service as before.
        result = service("ELG is an amazing project.")
        print(f"\\nResult:\\n{result}")

        # It is possible to use a file as input when running the service.
        with open("/tmp/example.txt", "w") as f:
            f.write("ELG is an amazing project.")

        result = service("/tmp/example.txt")
        print(f"\\nResult:\\n{result}")

        # You can apply a method to the result to extract the information needed. To do so, you have to pass a
        # callable object in the ouput_func parameter.
        service = Service.from_id(5228)
        pretty_result = service("Ich habe diesen Film geliebt. Die Schauspieler, das Drehbuch: alles von einem Meisterwerk.", output_func=lambda x: x.dict()["texts"][0]["content"])
        print("Translation to Finnish: ", pretty_result)

        # You can also set the output_func parameter to "auto" to extract the information needed automaticly.
        # This is not working for all the services.
        service = Service.from_id(5228)
        pretty_result = service("Ich habe diesen Film geliebt. Die Schauspieler, das Drehbuch: alles von einem Meisterwerk.", output_func="auto")
        print("Translation to Finnish: ", pretty_result)
    """

    def __init__(
        self,
        id: int,
        resource_name: str,
        resource_short_name: List[str],
        resource_type: str,
        entity_type: str,
        description: str,
        keywords: List[str],
        detail: str,
        licences: List[str],
        languages: List[str],
        country_of_registration: List[str],
        creation_date: str,
        last_date_updated: str,
        functional_service: bool,
        functions: List[str],
        intended_applications: List[str],
        views: int,
        downloads: int,
        size: int,
        service_execution_count: int,
        status: str,
        under_construction: bool,
        record: dict,
        auth_object: Authentication,
        auth_file: str,
        scope: str,
        domain: str,
        use_cache: bool,
        cache_dir: str,
    ):
        """
        Init a Service object with the service information
        """
        super().__init__(
            id=id,
            resource_name=resource_name,
            resource_short_name=resource_short_name,
            resource_type=resource_type,
            entity_type=entity_type,
            description=description,
            keywords=keywords,
            detail=detail,
            licences=licences,
            languages=languages,
            country_of_registration=country_of_registration,
            creation_date=creation_date,
            last_date_updated=last_date_updated,
            functional_service=functional_service,
            functions=functions,
            intended_applications=intended_applications,
            views=views,
            downloads=downloads,
            size=size,
            service_execution_count=service_execution_count,
            status=status,
            under_construction=under_construction,
            domain=domain,
            record=record,
        )
        if self.resource_type != "ToolService" and self.resource_type != "Tool/Service":
            raise NotServiceException(self.id)
        self._authenticate(
            auth_object=auth_object, auth_file=auth_file, scope=scope, use_cache=use_cache, cache_dir=cache_dir
        )
        self._get_elg_execution_location()

    @classmethod
    def from_id(
        cls,
        id: int,
        auth_object: Authentication = None,
        auth_file: str = None,
        scope: str = None,
        domain: str = None,
        use_cache: bool = True,
        cache_dir: str = "~/.cache/elg",
    ):
        """
        Class method to init a Service class from its id. You can provide authentication information through the
        auth_object or the auth_file attributes. If not authentication information is provided, the Authentication
        object will be initialized.

        Args:
            id (int): id of the service.
            auth_object (elg.Authentication, optional): elg.Authentication object to use. Defaults to None.
            auth_file (str, optional): json file that contains the authentication tokens. Defaults to None.
            scope (str, optional): scope to use when requesting tokens. Can be set to "openid" or "offline_access"
                to get offline tokens. Defaults to "openid".
            domain (str, optional): ELG domain you want to use. "live" to use the public ELG, "dev" to use the
                development ELG and another value to use a local ELG. Defaults to "live".
            use_cache (bool, optional): True if you want to use cached files. Defaults to True.
            cache_dir (str, optional): path to the cache_dir. Set it to None to not store any cached files. Defaults to "~/.cache/elg".

        Returns:
            elg.Service: Service object with authentication information.
        """
        # Use auth_object/file domain if auth_object/file given
        if auth_object is not None:
            assert isinstance(
                auth_object, Authentication
            ), f"auth_object ({type(auth_object)}) must be an Authentication object."
            auth_domain = auth_object.domain
            if domain is not None:
                print(
                    "Warning: domain argument saved in the authentication object will overwrite the "
                    "domain argument you put."
                )
            domain = auth_domain
        elif isinstance(auth_file, str):
            auth_domain = get_argument_from_json(auth_file, "domain")
            if domain is not None:
                print(
                    "Warning: domain argument saved in the authentication filename will overwrite the "
                    "domain argument you put."
                )
            domain = auth_domain
        # Use "live" domain by default
        domain = get_domain(domain=domain if domain is not None else "live")
        record = get_metadatarecord(id=id, domain=domain, use_cache=use_cache, cache_dir=cache_dir)
        result = map_metadatarecord_to_result(id=id, record=record)
        result["auth_object"] = auth_object
        result["auth_file"] = auth_file
        result["scope"] = scope
        result["domain"] = domain
        result["use_cache"] = use_cache
        result["cache_dir"] = cache_dir
        return cls(**result)

    @classmethod
    def from_entity(
        cls,
        entity: Entity,
        auth_object: str = None,
        auth_file: str = None,
        scope: str = None,
        use_cache: bool = True,
        cache_dir="~/.cache/elg",
    ):
        """
        Class method to init a Service class from an Entity object. You can provide authentication information through the
        auth_object or the auth_file attributes. If not authentication information is provided, the Authentication object will be initialized.

        Args:
            entity (elg.Entity): Entity object to init as a Service.
            auth_object (elg.Authentication, optional): elg.Authentication object to use. Defaults to None.
            auth_file (str, optional): json file that contains the authentication tokens. Defaults to None.
            scope (str, optional): scope to use when requesting tokens. Can be set to "openid" or "offline_access"
                to get offline tokens. Defaults to "openid".
            domain (str, optional): ELG domain you want to use. "live" to use the public ELG, "dev" to use the
                development ELG and another value to use a local ELG. Defaults to "live".
            use_cache (bool, optional): True if you want to use cached files. Defaults to True.
            cache_dir (str, optional): path to the cache_dir. Set it to None to not store any cached files. Defaults to "~/.cache/elg".

        Returns:
            elg.Service: Service object with authentication information.
        """
        if entity.record is None:
            entity.record = get_metadatarecord(
                id=entity.id, domain=entity.domain, use_cache=use_cache, cache_dir=cache_dir
            )
        parameters = entity.__dict__
        parameters["auth_object"] = auth_object
        parameters["auth_file"] = auth_file
        parameters["scope"] = scope
        parameters["use_cache"] = use_cache
        parameters["cache_dir"] = cache_dir
        return cls(**parameters)

    @classmethod
    def from_docker_image(
        cls,
        image: str,
        execution_location: str,
        port: int = 8000,
    ):
        """
        Class method to init a Service class from a Docker image ELG compatible.

        Args:
            image (str): name of the Docker image ELG compatible
            execution_location (str): url of the ELG compatible endpoint, e.g., `http://localhost:8000/process`.
            port (int, optional): port to use locally. Defaults to 8000.

        Returns:
            elg.Service: Service object with authentication information.
        """
        execution_location = urlparse(execution_location)
        image_port = execution_location.port
        image_port = int(image_port) if image_port is not None else 80
        print(
            "To call the service locally, the docker image needs to be running. ",
            f"Please run the following command: `docker run -p 127.0.0.1:{port}:{image_port} {image}`.",
            "When calling the service, you need to set the `sync_mode` parameter to True, otherwise an error will be raised.",
        )
        auth_object = Authentication(domain="local")
        auth_object.expires_time = time.gmtime(time.time() + ONE_HUNDRED_YEARS_IN_SECONDS)
        auth_object.refresh_expires_time = time.gmtime(time.time() + ONE_HUNDRED_YEARS_IN_SECONDS)
        record = {
            "service_info": {
                "elg_execution_location_sync": f"http://127.0.0.1:{port}{execution_location.path}",
                "elg_execution_location": "None",
            }
        }
        parameters = {
            "id": 0,
            "resource_name": image,
            "resource_short_name": image,
            "resource_type": "ToolService",
            "entity_type": None,
            "description": None,
            "keywords": None,
            "detail": None,
            "licences": None,
            "languages": None,
            "country_of_registration": None,
            "creation_date": None,
            "last_date_updated": None,
            "functional_service": None,
            "functions": None,
            "intended_applications": None,
            "views": None,
            "downloads": None,
            "size": None,
            "service_execution_count": None,
            "status": None,
            "under_construction": None,
            "record": record,
            "auth_object": auth_object,
            "auth_file": None,
            "scope": None,
            "domain": "local",
            "use_cache": None,
            "cache_dir": None,
        }
        return cls(**parameters)

    def _get_elg_execution_location(self):
        """
        Method to get the elg execution location information from the metadata record.
        """
        self.elg_execution_location_sync = get_information(
            id=self.id, obj=self.record, infos=["service_info", "elg_execution_location_sync"]
        )
        self.elg_execution_location = get_information(
            id=self.id, obj=self.record, infos=["service_info", "elg_execution_location"]
        )

    @need_authentication()
    def __call__(
        self,
        request_input: Union[str, List[str], Request] = None,
        request_type: str = "text",
        sync_mode: bool = False,
        timeout: int = None,
        check_file: bool = True,
        verbose: bool = True,
        output_func: Union[str, Callable] = lambda response: response,
        **kwargs,
    ) -> Union[dict, str]:
        """
        Method to call a service. You can enter a string input or the path to the file to process.
        The output is returned in JSON format.

        Args:
            request_input (Union[str, List[str], Request]): can be the text to process directly, the name of the file to process,
                a list of texts, or directly a Request object.
            request_type (str, optional): precise the type of the request. Can be "text", "structuredText", or "audio".
                It is only used if request_input is not a Request object. Defaults to "text".
            sync_mode (bool, optional): True to use the sync_mode. Defaults to False.
            timeout (int, optional): number of seconds before timeout. Defaults to None.
            check_file (bool, optional): True to check if request_input can be a file or not. Defaults to True.
            verbose (bool, optional): False to avoid print messages. Defaults to True.
            output_func (Union[str, Callable], optional): function applied to the service response. It can be used
                to extract only the content from the response. If set to 'auto', a generic extractive function will
                be used. Defaults to lambda response: response.
            kwargs: additional keyword arguments used to hide deprecated arguments

        Raises:
            ValueError: if a parameter is not correct.
            ElgException: can raise a specific Elg exception if the request to the service did not succeed.

        Returns:
            Union[dict, str]: service response in JSON format or as a string if output_func returns a string.
        """

        if "input_str" in kwargs:
            request_input = kwargs.pop("input_str")
            warnings.warn(
                "'input_str' is deprecated and will be removed in a future version. Use 'request_input' instead.",
                FutureWarning,
            )

        if isinstance(request_input, str):
            if check_file and len(request_input) < 256 and Path(request_input).is_file():
                if request_type == "text":
                    request = TextRequest.from_file(request_input)
                elif request_type == "structuredText":
                    request = StructuredTextRequest.from_file(request_input)
                elif request_type == "audio":
                    request = AudioRequest.from_file(request_input)
                else:
                    raise ValueError(
                        "Invalid value for request_type. It musts be 'text', 'structuredText', or 'audio'."
                    )
            else:
                if request_type != "text":
                    raise ValueError("request_input is plain text but request_type is not set to 'text'.")
                request = TextRequest(content=request_input)

        elif isinstance(request_input, List):
            if request_type != "structuredText":
                raise ValueError(
                    "request_input is a list of plain texts but request_type is not set to 'structuredText'."
                )
            request = StructuredTextRequest(texts=[{"content": str(text)} for text in request_input])

        elif isinstance(request_input, Request):
            request = request_input

        else:
            raise ValueError(
                "'request_input' musts be the text to process directly, the name of the file to process, a list of texts, or directly a Request object."
            )

        if isinstance(output_func, str) and output_func == "auto":
            output_func = lambda response: response.auto_content()
        if not callable(output_func):
            raise ValueError("output_func should be a callable object or 'auto'.")

        if verbose:
            print(f"Calling:\n\t[{self.id}] {self.resource_name}")
            print(f"with request:\n\t{request}\n")

        headers = {"Authorization": f"Bearer {self.authentication.access_token}"}
        if sync_mode:
            return self._call_sync(request=request, headers=headers, timeout=timeout, output_func=output_func)
        else:
            return self._call_async(request=request, headers=headers, timeout=timeout, output_func=output_func)

    @catch_requests_error
    def _call_sync(self, request: Request, headers: Dict, timeout: int, output_func: Callable) -> Union[Dict, str]:
        """
        Method used by __call__ to call a service synchronously.

        Args:
            request (Request): Request object.
            headers (Dict): headers for the call.
            timeout (int): number of seconds before timeout.
            output_func (Union[str, Callable], optional): function applied to the service response. It can be used
                to extract only the content from the response. If set to 'auto', a generic extractive function will
                be used. Defaults to lambda response: response.

        Returns:
            Union[Dict, str]: service response in JSON format or as a string if output_func returns a string.
        """
        if isinstance(request, AudioRequest):
            data = request.content
            headers["Content-Type"] = "audio/x-wav" if request.format == "linear16" else "audio/mpeg"
        elif isinstance(request, TextRequest) or isinstance(request, StructuredTextRequest):
            data = json.dumps(request.dict())
            headers["Content-Type"] = "application/json"
        else:
            raise ValueError(f"Request type {type(request)} not recognized.")
        response = requests.post(self.elg_execution_location_sync, headers=headers, data=data, timeout=timeout)
        ensure_response_ok(response)
        return output_func(get_response(response.json()))

    @catch_requests_error
    def _call_async(self, request: Request, headers: Dict, timeout: int, output_func: Callable) -> Union[Dict, str]:
        """
        Method used by __call__ to call a service asynchronously.

        Args:
            request (Request): Request object.
            headers (Dict): headers for the call.
            timeout (int): number of seconds before timeout.
            output_func (Union[str, Callable], optional): function applied to the service response. It can be used
                to extract only the content from the response. If set to 'auto', a generic extractive function will
                be used. Defaults to lambda response: response.

        Returns:
            Union[Dict, str]: service response in JSON format or as a string if output_func returns a string.
        """
        if isinstance(request, AudioRequest):
            data = request.content
            headers["Content-Type"] = "audio/x-wav" if request.format == "linear16" else "audio/mpeg"
        elif isinstance(request, TextRequest) or isinstance(request, StructuredTextRequest):
            data = json.dumps(request.dict())
            headers["Content-Type"] = "application/json"
        else:
            raise ValueError(f"Request type {type(request)} not recognized.")
        response = requests.post(self.elg_execution_location, headers=headers, data=data, timeout=timeout)
        ensure_response_ok(response)
        response = response.json()["response"]
        assert response["type"] == "stored"
        headers.pop("Content-Type")
        uri = response["uri"]
        response = requests.get(uri, headers=headers)
        waiting_time = time.time()
        while response.ok and "progress" in response.json().keys():
            sleep(1)
            response = requests.get(uri, headers=headers, timeout=timeout)
            if time.time() - waiting_time > (timeout if timeout is not None else float("inf")):
                raise requests.exceptions.Timeout("The service didn't respond before the end of the timeout.")
        ensure_response_ok(response)
        return output_func(get_response(response.json()))



from setuptools import find_packages, setup

setup(
    name="elg",
    version="0.4.3",
    author="ELG Technical Team",
    url="https://gitlab.com/european-language-grid/platform/python-client",
    author_email="contact@european-language-grid.eu",
    description="Use the European Language Grid in your Python projects",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="tools, sdk, language technology, europe, european, nlp",
    license="MIT",
    packages=find_packages(exclude=("tests",)),
    install_requires=[
        "requests>=2.25.1",
        "tqdm>=4.55.0",
        "pandas>=1.2.2",
        "loguru>=0.5",
        "pydantic>=1.8",
        "flask>=1.0.0",
        "Flask-JSON>=0.3.0",
        "docker>=5.0.0",
    ],
    entry_points={"console_scripts": ["elg=elg.cli.elg:main"]},
    python_requires=">=3.6.0",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)



import sys
from argparse import ArgumentParser
from typing import List, Union

from loguru import logger

from . import BaseELGCommand


def search_command_factory(args):
    return SearchCommand(
        domain=args.domain,
        entity=args.entity,
        search=args.search,
        resource=args.resource,
        function=args.function,
        languages=args.languages,
        license=args.license,
        limit=args.limit,
    )


class SearchCommand(BaseELGCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        search_parser = parser.add_parser("search", description="Get searchrmation of an ELG entity")
        search_parser.add_argument(
            "--domain",
            type=str,
            default="live",
            required=False,
            help="ELG domain you want to use",
        )
        search_parser.add_argument(
            "--entity",
            type=str,
            default="LanguageResource",
            required=False,
            help="type of the entity to search. Can be 'LanguageResource', 'Organization', or 'Project'",
            choices=["LanguageResource", "Organization", "Project"],
        )
        search_parser.add_argument(
            "--search",
            type=str,
            default=None,
            required=False,
            help="terms to use for the search request",
        )
        search_parser.add_argument(
            "--resource",
            type=str,
            default=None,
            required=False,
            help="type of the language resource. Only used when the entity is set to 'LanguageResource'. Can be 'Tool/Service', 'Lexical/Conceptual resource', 'Corpus', or 'Language description'",
            choices=["Tool/Service", "Lexical/Conceptual resource", "Corpus", "Language description"],
        )
        search_parser.add_argument(
            "--function",
            type=str,
            default=None,
            required=False,
            help="type of the function of the service. Only used when resource set to 'Tool/Service'",
        )
        search_parser.add_argument(
            "--languages",
            type=str,
            action="append",
            default=None,
            required=False,
            help="language filter for the search request. Multiple language can be set. The full name or the ISO639 code of the language can be used",
        )
        search_parser.add_argument("--license", type=str, default=None, required=False, help="license filter")
        search_parser.add_argument("--limit", type=int, default=100, required=False, help="limit number of results")
        search_parser.set_defaults(func=search_command_factory)

    def __init__(
        self,
        domain: str = "live",
        entity: str = "LanguageResource",
        search: str = None,
        resource: str = None,
        function: str = None,
        languages: Union[str, list] = None,
        license: str = None,
        limit: int = 100,
    ):
        self._domain = domain
        self._entity = entity
        self._search = search
        self._resource = resource
        self._function = function
        self._languages = languages
        self._license = license
        self._limit = limit

    def run(self):
        from pprint import pformat

        from ..catalog import Catalog

        try:
            catalog = Catalog(domain=self._domain)
            parameters = self.__dict__
            parameters.pop("_domain")
            logger.info(f"Search with the following parameters:\n\n{pformat(parameters, indent=2)}\n")
            results = catalog.search(
                entity=self._entity,
                search=self._search,
                resource=self._resource,
                function=self._function,
                languages=self._languages,
                license=self._license,
                limit=self._limit,
            )
            catalog._display_entities(results)
        except Exception as e:
            logger.error(f"Error during corpus search - {e}")
            sys.exit(1)



import inspect
import re
from pathlib import Path
from typing import Dict

import docker
from flask import Flask
from flask import request as flask_request
from flask_json import JsonError, as_json
from loguru import logger

from .model import AudioRequest, StructuredTextRequest, TextRequest
from .utils.docker import COPY_FOLDER, DOCKERFILE, ENTRYPOINT


class FlaskService:

    requirements = ["Flask", "Flask-JSON", "gunicorn", "elg", "docker", "flask", "flask_json"]

    def __init__(self, name):
        self.name = name
        self.app = Flask(name)
        # Don't add an extra "status" property to JSON responses - this would break the API contract
        self.app.config["JSON_ADD_STATUS"] = False
        # Don't sort properties alphabetically in response JSON
        self.app.config["JSON_SORT_KEYS"] = False
        self.app.add_url_rule("/process", "process", self.process, methods=["POST"])

    def run(self):
        self.app.run()

    @as_json
    def process(self):
        """
        Main request processing logic - accepts a JSON request and returns a JSON response.
        """
        data = flask_request.get_json()
        if data.get("type") == "text":
            request = TextRequest(**data)
        elif data.get("type") == "structuredText":
            request = StructuredTextRequest(**data)
        elif data.get("type") == "audio":
            request = AudioRequest(**data)
        else:
            self.invalid_request_error()
        logger.info(f"Call with the input: {request}")
        response = self.process_request(request)
        logger.info(f"Get response: {response}")
        response = {"response": response.dict()}
        logger.info(f"Return: {response}")
        return response

    def invalid_request_error(self):
        """
        Generates a valid ELG "failure" response if the request cannot be parsed
        """
        raise JsonError(
            status_=400, failure={"errors": [{"code": "elg.request.invalid", "text": "Invalid request message"}]}
        )

    def process_request(self, request):
        if request.type == "text":
            return self.process_text(request)
        elif request.type == "structuredText":
            return self.process_structured_text(request)
        elif request.type == "audio":
            return self.process_audio(request)
        self.invalid_request_error()

    def process_text(self, request):
        raise NotImplementedError()

    def process_structured_text(self, request):
        raise NotImplementedError()

    def process_audio(self, request):
        raise NotImplementedError()

    @classmethod
    def create_requirements(cls, requirements=[], path=None):
        if path == None:
            path = Path(inspect.getsourcefile(cls))
        else:
            path = Path(path)
        requirements = cls.requirements + requirements
        with open(path.parent / "requirements.txt", "w") as f:
            f.write("\n".join(set(requirements)))

    @classmethod
    def create_docker_files(cls, required_files=[], required_folders=[], path=None):
        if path == None:
            path = Path(inspect.getsourcefile(cls))
        else:
            path = Path(path)
        required_files = [path.name] + required_files
        required_folders = "\n".join(
            [COPY_FOLDER.format(folder_name=str(Path(folder))) for folder in required_folders]
        )
        service_script = path.name[:-3]  # to remove .py
        with open(path.parent / "docker-entrypoint.sh", "w") as f:
            f.write(ENTRYPOINT.format(service_script=service_script))
        with open(path.parent / "Dockerfile", "w") as f:
            f.write(DOCKERFILE.format(required_files=" ".join(required_files), required_folders=required_folders))

    @classmethod
    def docker_build_image(cls, tag: str, pull: bool = True, path=None, **kwargs):
        if path == None:
            path = Path(inspect.getsourcefile(cls))
        else:
            path = Path(path)
        client = docker.from_env()
        image, _ = client.images.build(path=str(path.parent), tag=tag, pull=pull, **kwargs)
        return image

    @classmethod
    def docker_push_image(cls, repository: str, tag: str, username: str = None, password: str = None, **kwargs):
        client = docker.from_env()
        if username is not None and password is not None:
            auth_config = {"username": username, "password": password}
            client.images.push(repository=repository, tag=tag, auth_config=auth_config, stream=True, **kwargs)
        client.images.push(repository=repository, tag=tag, stream=True, **kwargs)
        return

    @classmethod
    def docker_build_push_image(
        cls,
        repository: str,
        tag: str,
        pull: bool = True,
        username: str = None,
        password: str = None,
        build_kwargs: Dict = {},
        push_kwargs: Dict = {},
    ):
        cls.docker_build_image(tag=f"{repository}:{tag}", pull=pull, **build_kwargs)
        cls.docker_push_image(repository=repository, tag=tag, username=username, password=password, **push_kwargs)
        return None



from .AnnotationsResponse import AnnotationsResponse
from .AudioResponse import AudioResponse
from .ClassificationResponse import ClassesResponse, ClassificationResponse
from .TextsResponse import TextsResponse, TextsResponseObject
from .utils import get_response
